/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import java.awt.Graphics2D;
import java.awt.Image;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JLabel;

/**
 *
 * @author Emerson Ribeiro de Mello
 */
public class Jogador extends Elemento {

    protected int cima; // código da tecla para mover para cima
    protected int baixo;// código da tecla para mover para baixo
    protected int direita;// código da tecla para mover para direita
    protected int esquerda;// código da tecla para mover para esquerda
    protected int atirar;// código da tecla para atirar
    // as 2 linhas abaixo são usadas so' para depuracao
    private JLabel lbVidas;
    private List<Bomba> bombas = new ArrayList<Bomba>();
    private int tamanhoExplosao;
    private int limiteBombas;
    protected int vidas;
    protected boolean imortal;
    protected int tempoImortal;

    /**
     * 
     * @param pai referencia a area de Desenho
     * @param imagemNome - Nome da imagem que sera' usada para representar o jogador
     * @param bombaNome  - Nome da imagem que sera' usada para representar a bomba deste jogador
     * @param mX Linha na matriz
     * @param mY Coluna na matriz
     * @param c codigo da tecla que sera' usado para mover para cima
     * @param b codigo da tecla que sera' usado para mover para baixo
     * @param d codigo da tecla que sera' usado para mover para esquerda
     * @param e codigo da tecla que sera' usado para mover para direita
     * @param a codigo da tecla que sera' usado para atirar
     */
    public Jogador() {

    }

    public Jogador(JogoTimer pai, String imagemNome, String bombaNome,
            int mx, int my, int c, int b, int d, int e, int a, JLabel lbVidas) {
        super(pai, imagemNome, mx, my);
        this.icone = this.carregarImagem(this.imagemNome);
        this.cima = c;
        this.baixo = b;
        this.direita = d;
        this.esquerda = e;
        this.atirar = a;
        this.tamanhoExplosao = 1;
        this.limiteBombas = 1;
        this.vidas = 3;
        this.lbVidas = lbVidas;
        this.tempoImortal = this.velocidade * 2;
    }

    /**
     * Desenha a imagem no contexto gráfico do Pai
     * @param g 
     */
    @Override
    public void desenhar(Graphics2D g) {

        for (Bomba b : bombas) {
            b.desenhar(g);
        }
        // desenhando o icone do Jogador na Tela
        g.drawImage(this.icone, this.posX, this.posY, this.jogo);


        // atualizando os jLabel com as coordenadas. Inuteis para um jogo real
        // mas a ideia poderia ser aproveitada para atualizar placar, etc.
        this.atualizarDados();
    }

    /**
     * Atualiza coordenadas, posicao na matriz e a bomba
     */
    @Override
    public void atualizar() {
        int destinoLin = matLinha; // linha destino apos o movimento
        int destinoCol = matColuna; // coluna destino apos o movimento
        int destinoX = posX; // coordenada X destino na area de desenho
        int destinoY = posY; // coordenada Y destino na area de desenho

        if (jogo.framePrincipal.teclado.teclaApertada(this.cima)) {
            destinoY = posY - this.velocidade;
            destinoLin--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.baixo)) {
            destinoY = posY + this.velocidade;
            destinoLin++;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.esquerda)) {
            destinoX = posX - this.velocidade;
            destinoCol--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.direita)) {
            destinoX = posX + this.velocidade;
            destinoCol++;
        }

        if(tempoImortal <= 0) {
            this.imortal = false;
            tempoImortal = velocidade * 2;
        } else tempoImortal--;

        
        // se a celula destino estiver livre, mova o personagem para la'
        if (jogo.estaLivre(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;

            if (jogo.verificaBloco(matLinha, matColuna) == 10) {
                tamanhoExplosao++;
                jogo.cenario[matLinha][matColuna] = 2;
            } else if (jogo.verificaBloco(matLinha, matColuna) == 11) {
                limiteBombas++;
                jogo.cenario[matLinha][matColuna] = 2;
            } else if (jogo.verificaBloco(matLinha, matColuna) == 12) {
                if (Math.random() < 0.5) {
                    if (limiteBombas > 1) {
                        limiteBombas--;
                    }

                } else {
                    if (tamanhoExplosao > 1)
                        tamanhoExplosao--;
                }
                jogo.cenario[matLinha][matColuna] = 2;
            }
        }

        if (jogo.framePrincipal.teclado.teclaApertadaPrimeiraVez(this.atirar)) {
            if (bombas.size() < limiteBombas) {
                bombas.add(new Bomba(true, this.velocidade * 5, false, 0, this.posX, this.posY, jogo, imagemNome, this.posX / 40, this.posY / 40, this.tamanhoExplosao, this));
                jogo.cenario[matLinha][matColuna] = 3;
            }

        }

        for (int i = 0; i < bombas.size(); i++) {
            if (bombas.get(i).isBombaViva()) {
                bombas.get(i).setTempoDeVidaDaBomba(bombas.get(i).getTempoDeVidaDaBomba() - 10);
            }
            if (bombas.get(i).isExplodindo()) {
                bombas.get(i).setTempoDeExplosao(bombas.get(i).getTempoDeExplosao() - 10);
            }
            if (bombas.get(i).isExplodida()) {
                bombas.remove(i);
            }   
        }
    }

    public void atualizarDados() {
        this.lbVidas.setText("" + this.vidas);
    }
    public int getLimiteBombas() {
        return limiteBombas;
    }

    public void setLimiteBombas(int limiteBombas) {
        this.limiteBombas = limiteBombas;
    }

    public int getTamanhoExplosao() {
        return tamanhoExplosao;
    }

    public void setTamanhoExplosao(int tamanhoExplosao) {
        this.tamanhoExplosao = tamanhoExplosao;
    }

    public List<Bomba> getBombas() {
        return bombas;
    }

    public void setBombas(List<Bomba> bombas) {
        this.bombas = bombas;
    }
}
