/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import principal.JavaBomber;
import java.awt.event.ActionEvent;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Arrays;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.Timer;
import principal.FimDeJogo;

/**
 * Essa clase contém a lógica principal do Jogo. A atualização da tela é feita
 * através de o Timer do pacote Swing. Por este motivo, essa classe tem que
 * implementar a Interface ActionListener
 * Essa classe herda de Canvas, pois a classe Canvas é ideal para fazer desenhos
 * e permite usufrir o bufferStrategy para melhorar a taxa de atualização da tela
 * 
 * @author Emerson Ribeiro de Mello
 */
public class JogoTimer extends Canvas implements ActionListener {

    public JavaBomber framePrincipal; // frame principal do jogo
    protected Jogador jog1; // Jogador1
    protected Jogador jog2; // Jogador2
    private boolean pause = false;
    private int vitoriasJog1;
    private int vitoriasJog2;
    private Timer timer; // Timer responsável pela atualização do jogo
    public int larguraBloco; // Dimensao dos personabens, fundos. É 40
    public int colunas; // total de colunas na matriz do jogo
    public int linhas; // total de linhas na matriz do jogo
    private Image imgcenarios[]; // contém as imagens de fundo do cenário
    private int taxaDeAtualizacao; // a cada N milisegundos a tela é redesenhada
    // 0 = ferro
    // 1 = tijolo
    // 2 = vazio
    // 3 = bomba
    // 4 = 
    // 5 = 
    // 6 = 
    // 7 = 
    // 8 = 
    // 9 = bloco sorteado
    // 10 = poder fogo
    // 11 = poder bomba
    // 12 = poder caveira
    // 13 =
    // 14 =
    // 15 =
    private int cimaJog1;
    private int baixoJog1;
    private int esquerdaJog1;
    private int direitaJog1;
    private int atirarJog1;
    private int cimaJog2;
    private int baixoJog2;
    private int esquerdaJog2;
    private int direitaJog2;
    private int atirarJog2;
    public short[][] cenario;

    public JogoTimer(JavaBomber p) {
        super();
        this.framePrincipal = p;
        this.setBackground(Color.GRAY);

        this.setIgnoreRepaint(true); // para melhorar a velocidade do redesenho
        this.larguraBloco = 40;
        this.colunas = 17;
        this.linhas = 11;
        this.taxaDeAtualizacao = 120;
        this.imgcenarios = new Image[15];
        this.imgcenarios[0] = (new ImageIcon(this.getClass().getResource("/imagens/ferro.png"))).getImage();
        this.imgcenarios[1] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[10] = (new ImageIcon(this.getClass().getResource("/imagens/poder_fogo.png"))).getImage();
        this.imgcenarios[11] = (new ImageIcon(this.getClass().getResource("/imagens/poder_bomba.png"))).getImage();
        this.imgcenarios[12] = (new ImageIcon(this.getClass().getResource("/imagens/poder_caveira.png"))).getImage();

        this.cimaJog1 = KeyEvent.VK_UP;
        this.baixoJog1 = KeyEvent.VK_DOWN;
        this.esquerdaJog1 = KeyEvent.VK_LEFT;
        this.direitaJog1 = KeyEvent.VK_RIGHT;
        this.atirarJog1 = KeyEvent.VK_SPACE;
        this.cimaJog2 = KeyEvent.VK_W;
        this.baixoJog2 = KeyEvent.VK_S;
        this.esquerdaJog2 = KeyEvent.VK_A;
        this.direitaJog2 = KeyEvent.VK_D;
        this.atirarJog2 = KeyEvent.VK_1;

    }

    public short[][] cenarioInicial() {
        short[][] cenarioInicial = { //11x17 (linha,coluna)
            {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
            {0, 2, 2, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 2, 2, 0},
            {0, 2, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 2, 0},
            {0, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 0},
            {0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0},
            {0, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 0},
            {0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0},
            {0, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 0},
            {0, 2, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 9, 0, 2, 0},
            {0, 2, 2, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 2, 2, 0},
            {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
        };
        return cenarioInicial;
    }

    /**
     * Método que gera faz o cenario ser escolhido randomicamente. 
     * @return 1 ou 2. Sendo 1 para tijolo e 2 para espaço vazio.
     */
    public short random() {
        double porcentagem = 0.8;
        short resultado = 2;
        if (Math.random() < porcentagem) {
            resultado = 1;
        }
        return resultado;
    }

    /**
     * Método invocado para iniciar a partida
     */
    public void comecaOJogo() {
        // criando o Timer com taxa de atualização de N milisegundos
        this.timer = new Timer(taxaDeAtualizacao, this);
        // criando 2 buffer para acelerar a taxa de desenho
        createBufferStrategy(2);

        // KeyEvent possui codigos para as teclas. Por exemplo, para obter
        // o codigo da tecla 'seta para cima' basta invocar KeyEvent.VK_UP
        // Veja o uso do KeyEvent na criacao dos jogadores abaixo

        // criando Jogador 1
        this.jog1 = new Jogador(this, "azul.png", "bomba.png", 1, 1,
                cimaJog1, baixoJog1,
                direitaJog1, esquerdaJog1, atirarJog1, framePrincipal.lbVidasP1);

        // criando Jogador 2
        this.jog2 = new Jogador(this, "vermelho.png", "bomba.png", 15, 9,
                cimaJog2, baixoJog2,
                direitaJog2, esquerdaJog2, atirarJog2, framePrincipal.lbVidasP2);

        cenario = cenarioInicial();

        for (int i = 0; i < this.linhas; i++) {
            for (int j = 0; j < this.colunas; j++) {
                if (cenario[i][j] == 9) {
                    cenario[i][j] = random();
                }
            }
        }
        // Disparando o timer para começar a atualização do Loop principal do Jogo
        // Todo o conteúdo do método actionPerformed dessa classe será executado
        // a cada 60 milisegundos.
        this.timer.start();
    }

    public void recomecarJogo() {
        this.finalizar();
        this.comecaOJogo();
    }

    /**
     * Principal METODO do JOGO, pois e' aqui que fica a logica que fara':
     * - processamento da logica do jogo (morreu, disparou, etc..)
     * - atualizar as coordenadas dos elementos na tela para o proximo redesenho
     * - redesenhar os elementos na tela
     * 
     * Este método é invocado a cada N milisegundos pelo Timer
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        this.framePrincipal.teclado.poll(); // processa os eventos no teclado
        this.processarLogica(); // atualiza as coordenadas dos elementos
        this.renderizar(); // desenha os elementos no buffer de desenho
        this.desenhar(); // faz a troca do buffer de desenho pelo buffer de apresentacao

    }

    /**
     * Atualiza as coordenadas/estados de todos os elementos presentes no jogo
     * No caso, somente 2 jogadores, mas aqui pode-se colocar qualquer outro
     * elemento que venha a ser incluido no jogo
     */
    public void processarLogica() {
        //Chama o atualizar dos elementos
        if (this.jog1 != null) {
            this.jog1.atualizar();
        }
        if (this.jog2 != null) {
            this.jog2.atualizar();
        }

        if (this.jog1.vidas == 0) {
            JDialog vencedor = new FimDeJogo(this.framePrincipal, true, this, jog2.imagemNome);

            vencedor.setLocationRelativeTo(null);
            vencedor.setVisible(true);
            vencedor.requestFocus();
            this.vitoriasJog2++;
        }

        if (this.jog2.vidas == 0) {
            JDialog vencedor = new FimDeJogo(this.framePrincipal, true, this, jog1.imagemNome);
            vencedor.setLocationRelativeTo(null);
            vencedor.setVisible(true);
            vencedor.requestFocus();
            this.vitoriasJog1++;
        }
        if (this.jog1.vidas == 0 || this.jog2.vidas == 0) {
            finalizar();

        }

    }

    /**
     * Renderiza no buffer secundário todos os elementos já com suas coordenadas/estados
     * atualizados.
     */
    public void renderizar() {
        Graphics g = getBufferStrategy().getDrawGraphics();
        Graphics2D g2 = (Graphics2D) g;
        //Limpando a tela

        // Gerando um gradiente que ficara' como cor de fundo
        //GradientPaint gp = new GradientPaint(25, 25, Color.white,50, 50, Color.RED, true);
        //g2.setPaint(gp);

        // ou pintando com uma cor definida por RGB
        Color c = new Color(255, 210, 174);
        g2.setColor(c);
        g2.fillRect(0, 0, getWidth(), getHeight());

        // desenhar cenario de fundo
        this.renderizarCenario(g2);

        // desenhando os elementos nao nulos
        if (this.jog1 != null) {
            this.jog1.desenhar(g2);
        }
        if (this.jog2 != null) {
            this.jog2.desenhar(g2);
        }

        //liberando os contextos gráficos
        g.dispose();
        g2.dispose();

    }

    /**
     * Efetuando a troca das areas de desenho e apresentacao. 
     * No metodo redenrizar os elemtos graficos foram desenhados em um buffer
     * secundario, agora iremos fazer com que esse buffer secundario torne-se
     * primario, ou seja, que apareca de fato na tela
     */
    public void desenhar() {
        if (!getBufferStrategy().contentsLost()) {
            getBufferStrategy().show();
        }
    }

    /**
     * Fim de jogo/rodada
     */
    public void finalizar() {
        this.jog1 = null;
        this.jog2 = null;
        this.timer.stop();
    }

    /**
     * Desenha o cenário com as paredes de tijolos e de ferro
     * @param g contexto gráfico
     */
    public void renderizarCenario(Graphics2D g) {
        for (int i = 0; i < 11; i++) { // linhas
            for (int j = 0; j < 17; j++) { // colunas
                if (this.cenario[i][j] != 2) { // se não for vazio, desenhe
                    g.drawImage(this.imgcenarios[this.cenario[i][j]], this.larguraBloco * j, this.larguraBloco * i, this.framePrincipal);
                }
            }
        }
    }

    /**
     * Verifica se na posicao lin,col da matriz tem um espaço vazio
     * @param lin numero da linha
     * @param col numero da coluna
     * @return true se estiver vazio, false caso contrario
     */
    public boolean estaLivre(int lin, int col) {
        boolean retorno = false;
        if (cenario[lin][col] == 2) {
            retorno = true;
        }
        if (cenario[lin][col] == 10) {
            retorno = true;
        }
        if (cenario[lin][col] == 11) {
            retorno = true;
        }
        if (cenario[lin][col] == 12) {
            retorno = true;
        }
        return retorno;
    }

    public int verificaBloco(int linha, int coluna) {
        return cenario[linha][coluna];
    }

    public boolean isDestrutivel(int linha, int coluna) {
        if (cenario[linha][coluna] == 1) {
            return true;
        }
        if (cenario[linha][coluna] == 10) {
            return true;
        }
        if (cenario[linha][coluna] == 11) {
            return true;
        }
        if (cenario[linha][coluna] == 12) {
            return true;
        }
        if (cenario[linha][coluna] == 3) {
            return true;
        }
        return false;
    }

    /**
     * Indica em que posicao na tela encontrase o inicio dessa o retangulo que 
     * compreende uma celula
     * @param celula posicao na matriz que deseja-se descobrir
     * @return Coordenada na tela
     */
    public int celulaParaPixel(int celula) {
        return larguraBloco * celula;

    }

    public void pausarJogo() {
        if (!this.pause) {
            timer.stop();
            this.pause = true;
            framePrincipal.lbPause.setVisible(pause);
        } else {
            timer.start();
            this.pause = false;
            framePrincipal.lbPause.setVisible(pause);
        }
    }

    /**
     * Indica qual celula o pixel esta' contido na matriz
     * @param pixel
     * @return 
     */
    public int pixelParaCelula(int pixel) {
        return pixel / larguraBloco;
    }

    public int getAtirarJog1() {
        return atirarJog1;
    }

    public void setAtirarJog1(int atirarJog1) {
        this.atirarJog1 = atirarJog1;
    }

    public int getAtirarJog2() {
        return atirarJog2;
    }

    public void setAtirarJog2(int atirarJog2) {
        this.atirarJog2 = atirarJog2;
    }

    public int getBaixoJog1() {
        return baixoJog1;
    }

    public void setBaixoJog1(int baixoJog1) {
        this.baixoJog1 = baixoJog1;
    }

    public int getBaixoJog2() {
        return baixoJog2;
    }

    public void setBaixoJog2(int baixoJog2) {
        this.baixoJog2 = baixoJog2;
    }

    public int getCimaJog1() {
        return cimaJog1;
    }

    public void setCimaJog1(int cimaJog1) {
        this.cimaJog1 = cimaJog1;
    }

    public int getCimaJog2() {
        return cimaJog2;
    }

    public void setCimaJog2(int cimaJog2) {
        this.cimaJog2 = cimaJog2;
    }

    public int getDireitaJog1() {
        return direitaJog1;
    }

    public void setDireitaJog1(int direitaJog1) {
        this.direitaJog1 = direitaJog1;
    }

    public int getDireitaJog2() {
        return direitaJog2;
    }

    public void setDireitaJog2(int direitaJog2) {
        this.direitaJog2 = direitaJog2;
    }

    public int getEsquerdaJog2() {
        return esquerdaJog2;
    }

    public void setEsquerdaJog2(int esquerdaJog2) {
        this.esquerdaJog2 = esquerdaJog2;
    }

    public int getEsquerdaJog1() {
        return esquerdaJog1;
    }

    public void setEsquerdaJog1(int esquerdaJog1) {
        this.esquerdaJog1 = esquerdaJog1;
    }
}
